@@ -1,6 +1,7 @@
 import base64
 import hashlib
 import json
+import requests
 from typing import Any, Dict, List
 
 import ecdsa
@@ -13,9 +14,10 @@ class Transaction:
     """A Cyber transaction.
 
     After initialization, one or more token transfers can be added by
-    calling the `add_transfer()` method. Finally, call `get_pushable()`
+    calling the `add_transfer()`, `add_cyberlink` method. Then, call `get_pushable()`
     to get a signed transaction that can be pushed to the `POST /txs`
-    endpoint of the Cosmos REST API.
+    endpoint of the Cyber REST API or call `broadcast(LCD_API=<LCD_API>)` method
+    to get signed transaction and broadcast it with LCD_API.
     """
 
     def __init__(
@@ -48,8 +50,8 @@ class Transaction:
             "value": {
                 "from_address": privkey_to_address(self._privkey),
                 "to_address": recipient,
-                "amount": [{"denom": denom, "amount": str(amount)}],
-            },
+                "amount": [{"denom": denom, "amount": str(amount)}]
+            }
         }
         self._msgs.append(transfer)
 
@@ -63,6 +65,51 @@ class Transaction:
         }
         self._msgs.append(cyberlink)
 
+    def add_delegation(self, validator: str, amount: int, denom: str = "eul") -> None:
+        delegation = {
+            "type": "cosmos-sdk/MsgDelegate",
+            "value": {
+                "delegator_address": privkey_to_address(self._privkey),
+                "validator_address": validator,
+                "amount": {"denom": denom, "amount": str(amount)}
+            }
+        }
+        self._msgs.append(delegation)
+
+    def add_undelegation(self, validator:  str, amount: int, denom: str = "eul"):
+        undelegation = {
+            "type": "cosmos-sdk/MsgUndelegate",
+            "value": {
+                "delegator_address": privkey_to_address(self._privkey),
+                "validator_address": validator,
+                "amount": {"denom": denom, "amount": str(amount)}
+            }
+        }
+        self._msgs.append(undelegation)
+
+    def add_redelegation(self, validator_src: str, validator_dst: str, amount: int, denom: str = "eul"):
+        redelegation = {
+            "type": "cosmos-sdk/MsgBeginRedelegate",
+            "value": {
+                "delegator_address": privkey_to_address(self._privkey),
+                "validator_src_address": validator_src,
+                "validator_dst_address": validator_dst,
+                "amount": {"denom": denom, "amount": str(amount)}
+            }
+        }
+        self._msgs.append(redelegation)
+
+    def add_withdraw(self, validator: str):
+        withdraw = {
+            "type": "cosmos-sdk/MsgWithdrawDelegationReward",
+            "value": {
+                "delegator_address": privkey_to_address(self._privkey),
+                "validator_address": validator
+            }
+        }
+        self._msgs.append(withdraw)
+
+
     def get_pushable(self) -> str:
         pubkey = privkey_to_pubkey(self._privkey)
         base64_pubkey = base64.b64encode(pubkey).decode("utf-8")
@@ -87,6 +134,15 @@ class Transaction:
         }
         return json.dumps(pushable_tx, separators=(",", ":"))
 
+    def broadcast(self, LCD_API: str):
+        res = requests.post(url=LCD_API + '/txs', data=self.get_pushable())
+        if res.status_code == 200:
+            res = res.json()
+            return res
+        else:
+            raise Exception("Broadcact failed to run by returning code of {}".format(res.status_code))
+
+
     def _sign(self) -> str:
         message_str = json.dumps(self._get_sign_message(), separators=(",", ":"), sort_keys=True)
         message_bytes = message_str.encode("utf-8")
@@ -110,4 +166,4 @@ class Transaction:
             "memo": self._memo,
             "sequence": str(self._sequence),
             "msgs": self._msgs,
-        }
+        }
\ No newline at end of file
